@@ -42,13 +42,12 @@ use PDL::Pod::Parser;
 	  'Sig'     => 'Signature',
 	  'Opt'     => 'Options',
 	  'Usage'   => 'Usage',
-          'Bad'     => 'Bad value support',  
 	 );
 
 sub new {
   my ($type) = @_;
   my $parser = new PDL::Pod::Parser;
-  $parser->select("METHODS|OPERATORS|CONTRUCTORS|FUNCTIONS|NAME");
+  $parser->select("FUNCTIONS|NAME");
   $parser->{CURFUNC} = undef;
   $parser->{SYMHASH} = {};
   $parser->{INBLOCK} = 0;
@@ -98,7 +97,7 @@ sub command {
 
 sub check_for_mode {
   my ($this,$txt,$sep) = @_;
-  if ($txt =~ /^(sig|example|ref|opt|usage|bad|body)/i) {
+  if ($txt =~ /^(sig|example|ref|opt|usage|body)/i) {
     $this->{Parmode} = ucfirst lc $1;
     print "switched now to '$1' mode\n" if $this->{VERBOSE};
     print "\n\t$Title{$this->{Parmode}}\n"
@@ -169,7 +168,7 @@ sub trim {
 
 =head1 NAME
 
-PDL::Doc - support for PDL online documentation
+  PDL::Doc -- support for PDL online documentation
 
 =head1 SYNOPSIS
 
@@ -214,10 +213,9 @@ Individual functions in this section are introduced by
 
 
 =pod
-
 where signature is the argumentlist for a PP defined function
 as explained in L<PDL::PP>. Generally, PDL documentation is in
-valid POD format (see L</perlpod>) but uses the C<=for> directive
+valid POD format (see L<perlpod>) but uses the C<=for> directive
 in a special way. The C<=for> directive is used to flag to the
 PDL Pod parser that information is following that will be used
 to generate online help.
@@ -286,20 +284,12 @@ gives examples of typical usage for the current function:
          $_->wpic($name[0],{CONVERTER => 'ppmtogif'})
        }
 
-=item Bad
-
-provides information on how the function handles bad values (if
-C<$PDL:Config{WITH_BADVAL}> is set to 1). The intention is to
-have this information automatically created for pp-compiled
-functions, although it can be over-ridden.
-
 =back
 
 The PDL podparser is implemented as a simple state machine. Any of
 the above C<=for> statements switches the podparser into a state
 where the following paragraph is accepted as information for the
-respective field (C<Ref>, C<Usage>, C<Opt>, C<Example> or C<Bad>). 
-Only the text up to
+respective field (C<Ref>, C<Usage>, C<Opt> or C<Example>). Only the text up to
 the end of the current paragraph is accepted, for example:
 
   =for example
@@ -397,10 +387,7 @@ has been introduced into the definition of C<pp_def> (see again L<PDL::PP>)
 which will take care that name and signature of the so defined function
 are documented in this way (for examples of this usage see, for example,
 the PDL::Slices module, especially F<slices.pd> and the resulting
-F<Slices.pm>). Similarly, the 'BadDoc' field provides a means of
-specifying information on how the routine handles the presence of
-bad values: this will be autpmatically created if 
-C<BadDoc> is not supplied, or set to C<undef>.
+F<Slices.pm>).
 
 Furthermore, the documentation for each function should contain
 at least one of the I<Usage> or I<Examples> fields. Depending on the
@@ -551,13 +538,11 @@ sub search {
   $sort = 0 unless defined $sort;
   my $hash = $this->ensuredb;
   my @match = ();
-
   $pattern = $this->checkregex($pattern);
-
   while (my ($key,$val) = each %$hash) {
     for (@$fields) {
-      if (($_ eq 'Name' && $key =~ /$pattern/i) ||
-	  (defined $val->{$_} && $val->{$_} =~ /$pattern/i)) {
+      if (($_ eq 'Name' && $key =~ /$pattern/) ||
+	  (defined $val->{$_} && $val->{$_} =~ /$pattern/)) {
 	$val = $hash->{$val->{Crossref}}
 	  if defined $val->{Crossref} && defined $hash->{$val->{Crossref}};
 	push @match, [$key,$val];
@@ -575,14 +560,13 @@ sub search {
 # where the pairs of '/' can be replaced by any other pair of matching
 # characters
 # if the expression doesn't start with 'm' followed by a nonalphanumeric
-# character,  return as-is
+# character return as is
 sub checkregex {
   my ($this,$regex) = @_;
   return "(?i)$regex" unless $regex =~ /^m[^a-z,A-Z,0-9]/;
   my $sep = substr($regex,1,1);
   substr($regex,0,2) = '';
-  $sep = '(?<!\\\\)\\'.$sep; # Avoid '\' before the separator 
-
+  $sep = '\\'.$sep; # quote separator just in case
   my ($pattern,$mod) = split($sep,$regex,2);
   barf "unknown regex modifiers '$mod'" if $mod && $mod !~ /[imsx]+/;
   $pattern = "(?$mod)$pattern" if $mod;
@@ -654,19 +638,12 @@ sub scan {
   my @namelines = split("\n",$outfile->{Text});
   my ($name,$does);
   for (@namelines) {
-     if (/^(PDL) (-) (.*)/ or /\s*(PDL::[\w:]*)\s*(-*)?\s*(.*)\s*$/) {
+     if (/^(PDL) (-) (.*)/ or /\s*(PDL::[\w:]*)\s*(-*)?\s*(.*)\s*$/){
        $name = $1; $does = $3;
      }
-     if (/^\s*([a-z]+) (-+) (.*)/) { # lowercase shell script name
-       $name = $1; $does = $3;
-       ($name,$does) = (undef,undef) unless $does =~ /shell|script/i;
-     }
    }
    $does = 'Hmmm ????' if $does =~ /^\s*$/;
-   my $type = ($file =~ /\.pod$/ ? 
-	       ($does =~ /shell|script/i && $name =~ /^[a-z]+$/) ? 'Script:' 
-	       : 'Manual:'
-	       : 'Module:');
+   my $type = ($file =~ /\.pod$/ ? 'Manual:' : 'Module:');
    $hash->{$name} = {Ref=>"$type $does",File=>$file2} if $name !~ /^\s*$/;
    return $n;
 }
@@ -688,9 +665,8 @@ sub scantree {
   my $sub = sub { if (($File::Find::name =~ /[.]pm$/ &&
 		      $File::Find::name !~ /PP.pm/ &&
 		      $File::Find::name !~ m|Pod/Parser.pm| &&
-		      $File::Find::dir !~ m#/PP|/Gen#) or (
-		       $File::Find::name =~ /[.]pod$/ && 
-                       $File::Find::name !~ /Index[.]pod$/)){
+		      $File::Find::dir !~ m#/PP|/Gen#) or
+		       $File::Find::name =~ /[.]pod$/){
        printf "%-20s", $_.'...';
        my $n = $this->scan($File::Find::name,$verbose); # bind $this lexically
        print "\t$n functions\n";
@@ -745,12 +721,8 @@ sub extrdoc {
 sub getfuncdocs {
   my ($func,$in,$out) = @_;
   my $parser = new PDL::Pod::Parser;
-#  $parser->select("\\(METHODS\\|OPERATORS\\|CONSTRUCTORS\\|FUNCTIONS\\|METHODS\\)/$func(\\(.*\\)*\\s*");
-  foreach my $foo(qw/FUNCTIONS OPERATORS CONSTRUCTORS METHODS/) {
-      seek $in,0,0;
-      $parser->select("$foo/$func(\\(.*\\))*\\s*");
-      $parser->parse_from_filehandle($in,$out);
-  }
+  $parser->select("FUNCTIONS/$func(\\(.*\\))*\s*");
+  $parser->parse_from_filehandle($in,$out);
 }
 
 1;
@@ -762,14 +734,11 @@ discussions on the pdl-porters mailing list.
 
 =head1 AUTHOR
 
-Copyright 1997 Christian Soeller E<lt>c.soeller@auckland.ac.nzE<gt> 
-and Karl Glazebrook E<lt>kgb@aaoepp.aao.gov.auE<gt>
+Copyright 1997 Christian Soeller <c.soeller@auckland.ac.nz> and Karl Glazebrook
+<kgb@aaoepp.aao.gov.au>
 All rights reserved. There is no warranty. You are allowed
 to redistribute this software / documentation under certain
 conditions. For details, see the file COPYING in the PDL
 distribution. If this file is separated from the PDL distribution,
 the copyright notice should be included in the file.
 
-=cut
-
-1;
