@@ -87,8 +87,7 @@ give header information, rather than using a .hdr file.  For example,
 	];
 	@a = readflex('banana',$header);
 
-reads our example file again.  As a special case, when NDims is 1, Dims
-may be given as a scalar.
+reads our example file again.
 
 Within PDL, readflex and writeflex can be used to write several pdls
 to a single file -- e.g.
@@ -111,31 +110,6 @@ The reading of compressed data is switched on automatically if the
 filename requested ends in .gz or .Z, or if the originally specified
 filename does not exist, but one of these compressed forms does.
 
-If writeflex and readflex are given a reference to a file handle as a
-first parameter instead of a filename, then the data is read or
-written to the open filehandle. This gives an easy way to read an
-arbitrary slice in a big data volume, as in the following example:
-
-	use PDL;
-	use PDL::IO::FastRaw;
-
-        open(DATA, "raw3d.dat");
-        binmode(DATA);
-
-        # assume we know the data size from an external source
-        ($width, $height, $data_size) = (256,256, 4);
-
-        my $slice_num = 64;   # slice to look at
-        # Seek to slice
-        seek(DATA, $width*$height*$data_size * $slice_num, 0);
-        $pdl = readflex \*DATA, [{Dims=>[$width, $height], Type=>'long'}];
-
-WARNING: In later versions of perl (5.8 and up) you must be sure that your file
-is in "raw" mode (see the perlfunc man page entry for "binmode", for
-details).  Both readflex and writeflex automagically switch the file
-to raw mode for you -- but in code like the snipped above, you could
-end up seeking the wrong byte if you forget to make the binmode() call.
-
 Mapflex memory maps, rather than reads, the data files.  Its interface
 is similar to `readflex'.  Extra options specify if the data is to be
 loaded `ReadOnly', if the data file is to be `Creat'-ed anew on the
@@ -182,7 +156,6 @@ Read a binary file with flexible format specification
 =for usage
 
  ($x,$y,...) = readflex("filename" [, $hdr])
- ($x,$y,...) = readflex(FILEHANDLE [, $hdr])
 
 
 =head2 writeflex
@@ -194,7 +167,6 @@ Write a binary file with flexible format specification
 =for usage
 
   $hdr = writeflex($file, $pdl1, $pdl2,...)
-  $hdr = writeflex(FILEHANDLE, $pdl1, $pdl2,...)
 
 
 =head2 mapflex
@@ -226,7 +198,7 @@ package PDL::IO::FlexRaw;
 use PDL;
 use Exporter;
 use FileHandle;
-use PDL::Types ':All';
+use PDL::Types;
 use PDL::IO::Misc qw(bswap2 bswap4 bswap8);
 
 @PDL::IO::FlexRaw::ISA = qw/Exporter/;
@@ -234,36 +206,19 @@ use PDL::IO::Misc qw(bswap2 bswap4 bswap8);
 @EXPORT = qw/writeflex writeflexhdr readflex mapflex/;
 
 # Cast type numbers in concrete, for external file's sake...
-%flexnames = ( map {(typefld($_,'numval') => typefld($_,'ioname'))}
-	       typesrtkeys());
-%flextypes = ( map {(typefld($_,'ioname') => typefld($_,'numval'),
-		     typefld($_,'numval') => typefld($_,'numval'),
-		     lc typefld($_,'ppsym') =>  typefld($_,'numval'),
-		    )}
-	       typesrtkeys());
-%flexswap = ( map {my $val = typefld($_,'numval');
-		   my $nb = PDL::Core::howbig($val);
-		   ($val =>  $nb > 1 ? "bswap$nb" : undef)}
-	      typesrtkeys());
+%flexnames = (
+   $PDL_B => 'byte', $PDL_S => 'short',
+   $PDL_US => 'ushort', $PDL_L => 'long',
+   $PDL_F => 'float', $PDL_D => 'double');
 
-# use Data::Dumper;
-# print Dumper \%flexnames;
-# print Dumper \%flextypes;
-# print Dumper \%flexswap;
-
-# %flexnames = (
-#    $PDL_B => 'byte', $PDL_S => 'short',
-#    $PDL_US => 'ushort', $PDL_L => 'long',
-#    $PDL_F => 'float', $PDL_D => 'double');
-
-# %flextypes = (
-# 'byte'   => $PDL_B, '0' => $PDL_B, 'b' => $PDL_B, 'c' => $PDL_B,
-# 'short'  => $PDL_S, '1' => $PDL_S, 's' => $PDL_S,
-# 'ushort' => $PDL_US,'2' => $PDL_US,'u' => $PDL_US,
-# 'long'   => $PDL_L, '3' => $PDL_L, 'l' => $PDL_L,
-# 'float'  => $PDL_F, '4' => $PDL_F, 'f' => $PDL_F,
-# 'double' => $PDL_D, '5' => $PDL_D, 'd' => $PDL_D
-# );
+%flextypes = (
+'byte'   => $PDL_B, '0' => $PDL_B, 'b' => $PDL_B, 'c' => $PDL_B,
+'short'  => $PDL_S, '1' => $PDL_S, 's' => $PDL_S,
+'ushort' => $PDL_US,'2' => $PDL_US,'u' => $PDL_US,
+'long'   => $PDL_L, '3' => $PDL_L, 'l' => $PDL_L,
+'float'  => $PDL_F, '4' => $PDL_F, 'f' => $PDL_F,
+'double' => $PDL_D, '5' => $PDL_D, 'd' => $PDL_D
+);
 
 $PDL::FlexRaw::verbose = 0;
 
@@ -335,7 +290,7 @@ sub readchunk {
     my ($d, $pdl, $len, $name) = @_;
     print "Reading $len at $offset from $name\n"
       if $PDL::FlexRaw::verbose;
-    read($d, ${$pdl->get_dataref}, $len) == $len
+    $d->read($ {$pdl->get_dataref},$len) == $len
 	or barf "Couldn't read enough data from '$name'";
     $pdl->upd_data();
     $offset += $len;
@@ -362,20 +317,13 @@ sub mapchunk {
 }
 
 sub readflex {
-    barf 'Usage ($x,$y,...) = readflex("filename"|FILEHANDLE [, \@hdr])'
+    barf 'Usage ($x,$y,...) = readflex("filename" [, \@hdr])'
 	if $#_ > 1;
     my ($name,$h) = @_;
     my ($hdr, $pdl, $len, @out, $chunk, $chunkread, $data);
     local ($offset) = 0;
     my ($newfile, $swapbyte, $f77mode, $zipt) = (1,0,0,0);
-    my $d;
 
-    # Test if $name is a file handle
-    if (defined fileno($name)) {
-	$d = $name;
-	binmode($d); 
-    }
-    else {
     if ($name =~ s/\.gz$// || $name =~ s/\.Z$// ||
 	(! -e $name && (-e $name.'.gz' || -e $name.'.Z'))) {
 	$data = "gzip -dcq $name |";
@@ -385,13 +333,12 @@ sub readflex {
     }
 
     my ($size) = (stat $name)[7];
-	$d = new FileHandle $data
+    my ($d) = new FileHandle $data
 	or barf "Couldn't open '$data' for reading";
     binmode $d;
     if ($#_ == 0) {
 	$h = _read_flexhdr("$name.hdr");
     }
-    }
 
 # Go through headers which reconfigure
     foreach $hdr (@$h) {
@@ -429,18 +376,16 @@ READ:
 	    $type = $flextypes{$type};
 	}
 	$pdl = PDL->zeroes ((new PDL::Type($type)),
-			    ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims});
+			    @{$hdr->{Dims}});
 	$len = length $ {$pdl->get_dataref};
 
 	&readchunk($d,$pdl,$len,$name) or last READ;
 	$chunkread += $len;
 	if ($swapbyte) {
-	  my $method = $flexswap{$type};
-	  $pdl->$method if $method;
-# 	    bswap2($pdl) if $pdl->get_datatype == $PDL_S;
-# 	    bswap4($pdl) if $pdl->get_datatype == $PDL_L
-# 		|| $pdl->get_datatype == $PDL_F;
-# 	    bswap8($pdl) if $pdl->get_datatype == $PDL_D;
+	    bswap2($pdl) if $pdl->get_datatype == $PDL_S;
+	    bswap4($pdl) if $pdl->get_datatype == $PDL_L
+		|| $pdl->get_datatype == $PDL_F;
+	    bswap8($pdl) if $pdl->get_datatype == $PDL_D;
 	}
 	if ($newfile && $f77mode) {
 	    if ($zipt || $swapbyte) {
@@ -494,7 +439,7 @@ READ:
 sub mapflex {
     my ($usage)
 	= 'Usage ($x,$y,...) = mapflex("filename" [, \@hdr] [,\%opts])';
-    my $name = shift;
+    my ($name) = shift @_;
     # reference to header array
     my ($h, $size);
     # reference to options array, with defaults
@@ -510,7 +455,7 @@ sub mapflex {
 	} elsif (ref($_) eq "HASH") {
 	    %opts = (%opts,%$_);
 	} else {
-	    warn $usage;
+	    barf $usage;
 	}
     }
 
@@ -532,7 +477,7 @@ sub mapflex {
 	    $f77mode = 1;
 	} else {
 	    my($si) = 1;
-	    foreach (ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims}) {
+	    foreach (@{$hdr->{Dims}}) {
 		$si *= $_;
 	    }
 	    $size += $si * PDL::Core::howbig ($type);
@@ -550,8 +495,6 @@ sub mapflex {
 
     $d = byte PDL->zeroes(1);
     # print "Mapping total size $size\n";
-    # use Data::Dumper;
-    # print "Options: ", Dumper(\%opts), "\n";
     $d->set_data_by_mmap($name,$size,1,($opts{ReadOnly}?0:1),
 			 ($opts{Creat}?1:0),
 			 (0644),
@@ -579,7 +522,7 @@ READ:
 	    $type = $flextypes{$type};
 	}
 	$pdl = PDL->zeroes ((new PDL::Type($type)),
-			    ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims});
+			    @{$hdr->{Dims}});
 	$len = length $ {$pdl->get_dataref};
 
 	&mapchunk($d,$pdl,$len,$name) or last READ;
@@ -616,24 +559,14 @@ READ:
 }
 
 sub writeflex {
-    my $usage = 'Usage writeflex("filename"|FILEHANDLE,$pdl,...)';
-    barf $usage if $#_<0;
+    barf 'Usage writeflex("filename",$pdl,...)' if $#_<0;
     my($name) = shift; my (@ret);
-    my $d;
-
-    # Test if $name is a file handle
-    if (defined fileno($name)) {
-	$d = $name;
-	binmode $d;
-    }
-    else {
-	barf $usage if ref $name;
-	$d = new FileHandle ">$name"
+    barf 'Usage writeflex("filename",$pdl,...)' if ref $name;
+    my $d = new FileHandle ">$name"
 	or barf "Couldn't open '$name' for writing";
     binmode $d;
-    }
     foreach $pdl (@_) {
-	barf $usage if ! ref $pdl;
+	barf 'Usage writeflex("filename",$pdl,...)' if ! ref $pdl;
 	# print join(' ',$pdl->getndims,$pdl->dims),"\n";
 	push @ret, {
 	    Type => $flexnames{$pdl->get_datatype},
@@ -661,8 +594,7 @@ sub writeflexhdr {
 	}
 	print $h join("\n",$_->{Type},
 		      $_->{NDims},
-		      (join ' ',ref $_->{Dims} ? @{$_->{Dims}} : $_->{Dims})),
-	"\n\n";
+		      (join ' ',@{ $_->{Dims}})),"\n\n";
     }
 }
 
